#!/usr/bin/perl

# author: Kevin K.M. Chiu
#       Cobalt Networks
# version: 3.0
#
# Version 2.0 uses Monitor.pm instead of AlertDB.pm
#
# date: 6-28-99
#
# Copyright 1999-2000, Cobalt Networks, Inc
# All rights reserved

### START HARDCODED PATHS
my $Template_dir = '/usr/admserv/templates';
my %defcon_images = ('0','<IMG SRC="/.cobalt/images/transparentball" BORDER=0 WIDTH=20 HEIGHT=15>','1','<IMG SRC="/.cobalt/images/greenball" BORDER=0 WIDTH=20 HEIGHT=15>','2','<IMG SRC="/.cobalt/images/yellowball" BORDER=0 WIDTH=20 HEIGHT=15>','3','<IMG SRC="/.cobalt/images/redball" BORDER=0 WIDTH=20 HEIGHT=15>');

my $Debug = (-e "/etc/DEBUG") ? 1 : 0;
$^W = $Debug;
### END HARDCODED PATHS
use Cobalt::Locale;
use Cobalt::swatch;
use Cobalt::RAID;

# use strict;
use CGI;
use Locale::gettext;

# global variables
my $q=new CGI;

# the page that run this script
my $page=$q->param('page');

my $alertScriptURL='/cgi-bin/.cobalt/alert/alert.cgi';

my $locale=Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
if( $locale ) { $ENV{ 'LANGUAGE' }=$locale->getLocale(); }

# get ball colors
my (%balls,$defcon,$obj,@items,$comp,$service,$item);

# cpu
$balls{"cpu"} = $defcon_images{ swatch_get_defcon("cpu") };
# memory
$balls{"memory"} = $defcon_images{ swatch_get_defcon("memory") };
# network
$balls{"network"} = $defcon_images{ 1 };
# disk usage
$balls{"disk"} = $defcon_images{ swatch_get_defcon_disk() };

# RAID "disk integrity"
my $raid_enabled;
unless (-e '/etc/cobalt/ha-config') {
    open(MOUNTS, "/bin/mount |");
    while(<MOUNTS>) {
        if(/md1 on \/ /) {
            $raid_enabled = 1;
            last;
        }
    }
    close(MOUNTS);
}

my $raid_line;
if ($raid_enabled) {
    my $raid_state = (&raid_get_state)[0];
    $raid_state ||= 3;
    $raid_state = 3 if ($raid_state == 4); # generalize error state
    $raid_state = 3 if ($raid_state == 5); # ditto
    $balls{"raid"} = $defcon_images{ $raid_state };

    $raid_line = <<EOF;
        <TR>
                <TD WIDTH="25"><P ALIGN="CENTER">[RAID]</TD>
                <TD WIDTH="375"><FONT COLOR=#FFFFFF>&nbsp;&nbsp;</FONT><A HREF="/cgi-bin/.cobalt/raidUsage/raidUsage.cgi?alert=t">[str.diskraid]</A></TD>
        </TR>
EOF
} 

# now do services
my @services = ("http", "ftp", "snmp", "dns", "telnet");
foreach $service (@services) {
    $balls{ $service } = $defcon_images{ swatch_get_defcon("service", $service) };
}
# don't forget email
$balls{"email"} = $defcon_images{ swatch_get_defcon_email() };

# Read in the template
open( TEMPLATE, "$Template_dir/alert.tmpl" )
    || die "Cannot read $Template_dir/alert.tmpl: $!\n";

print "content-type: text/html\nCache-Control: no-cache\n\n";

while ( <TEMPLATE> ) {
    s/\[RAIDLINE\]/$raid_line/o;

    s/\[CPU\]/$balls{"cpu"}/o;
    s/\[MEMORY\]/$balls{"memory"}/o;
    s/\[DISK\]/$balls{"disk"}/o;
    s/\[RAID\]/$balls{"raid"}/o;
    s/\[NETWORK\]/$balls{"network"}/o;

    s/\[FTP\]/$balls{"ftp"}/o;
    s/\[EMAIL\]/$balls{"email"}/o;
    s/\[TELNET\]/$balls{"telnet"}/o;
    s/\[HTTP\]/$balls{"http"}/o;
    s/\[SNMP\]/$balls{"snmp"}/o;
    s/\[DNS\]/$balls{"dns"}/o;

    s/\[DEFCON0\]/$defcon_images{ 0 }/o;
    s/\[DEFCON1\]/$defcon_images{ 1 }/o;
    s/\[DEFCON2\]/$defcon_images{ 2 }/o;
    s/\[DEFCON3\]/$defcon_images{ 3 }/o;

    s/\[str.([^\]]+)\]/Locale::gettext::dgettext("ui", "$1")/geo;

    print;
}

close TEMPLATE;
