#!/usr/bin/perl
#
# Adrian Sun <asun@cobaltnet.com>
# Copyright 1999 Cobalt Networks http://www.cobaltnet.com/
#
# inspired by siteUserList.cgi

### START HARDCODED PATHS
my $Htgroup_dir = '/home/sites';
my $Template_dir = '/usr/admserv/templates';
my $Vol_diskdir = '/mnt/disk';
my $Vol_home = 'home';
my $Vol_rootdir = '/vol';

my $Debug = (-e "/etc/DEBUG") ? 1 : 0;
$^W = $Debug;
### END HARDCODED PATHS
use Cobalt::DiskUtils;
use Cobalt::Util;
use Cobalt::Locale;

use CGI;

use Locale::gettext;
Locale::gettext::textdomain("ui");

my $info;

my $q = new CGI;
my $page = $q->param('page');

if( $ENV{ PARAM_INFO } ) {
    $info=$ENV{ PARAM_INFO };
} else {
    $info=$q->param( 'info' );
}

my $diskListPageURL ="/cgi-bin/.cobalt/disk/diskList.cgi";
my $diskModPageURL ="/cgi-bin/.cobalt/disk/diskMod.cgi";

my $locale=Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
if( $locale ) { $ENV{ 'LANGUAGE' }=$locale->getLocale(); }

my $error_noent = MSG_get_ui("volErrNoEnt");
my $error_badname = MSG_get_ui("volErrBadName");
my $error_exist = MSG_get_ui("volErrExist");
my $error_busy = MSG_get_ui("volErrBusy");
my $error_mountpoint = MSG_get_ui("volErrMountPoint");


if ($page eq 'diskMod' and not ($info = handleDiskModify($q))) {
    print "location: $diskListPageURL\n\n";
} else {
    print "content-type: text/html\n";
    print "Cache-Control: no-cache\n\n";
    printDiskModifyPage( \*STDOUT, $q, $info);
}
1;


sub handleDiskModify
{
    my $q = shift;

    my $partition = $q->param('partition');
    my $oldenabled = $q->param('state');
    my $oldname = $q->param('label');
    my $name = $q->param('volname');
    my $enabled = $q->param('enabled') ? "enabled" : "disabled";
    
    return $error_noent unless (defined($name) and $name ne "");
    return $error_badname unless $name =~ /^[\w\d\-\.]+$/;
    return $error_exist if $name eq $Vol_home;

    if ($name ne $oldname) {
	my $newdev = "$Vol_rootdir/$name";
	my $flags = "override,nocheck";

	$flags .= ",disable" if $enabled eq "disabled";

	return $error_exist if -d $newdev;

	return $error_busy if 
	    ($oldenabled eq "enabled") and disk_umount($partition);

	# fix up the disk and unmount it if it's disabled.
	disk_mount($partition, $flags, $name);
	disk_umount($newdev) if $enabled eq "disabled";

	# cleanup
	rmdir("$Vol_rootdir/$oldname");

	# retarget all of the sites that use this disk. 
	opendir(DIR, $Htgroup_dir);
	while ($_ = readdir(DIR)) {
	    my ($sitename, $link);

	    next unless /^site/;
	    $sitename = "$Htgroup_dir/$_";
	    next unless (-l $sitename);

	    $link = readlink($sitename);
	    next unless $link =~ /^$Vol_rootdir\/$oldname/o;
	    unlink($sitename);
	    symlink("$newdev/sites/$_", $sitename);
	}
	closedir(DIR);
	return undef;
    }
    
    if ($enabled ne $oldenabled) {
	if ($enabled eq "enabled") {
	    return $error_mountpoint unless 
		(disk_mountok($Vol_diskdir) and disk_mountok($Vol_rootdir));
	    return $error_mountpoint unless disk_mount($partition, "enable");
	} else {
	    return $error_busy if disk_umount($partition, "disable");
	}
    }
    
    return undef;
}


# things that can be modified
sub printDiskModifyPage
{

    my ($handle, $q, $info) = @_;
    my $partition = $q->param('partition');
    my $label = $q->param('label');
    my $state = $q->param('state');

    my $enabled = "CHECKED" if ($state eq "enabled");

    open(TEMPLATE, "$Template_dir/diskMod.tmpl") or 
	die "Can't read $Template_dir/diskMod.tmpl: $!\n";

    while (<TEMPLATE>) {
	s/\[PARTITION\]/$partition/o;
	s/\[VOLNAME\]/$label/o;
	s/\[STATE\]/$state/o;
	s/\[ENABLE_STATE\]/$enabled/o;
	s/\[DISK_INFO\]/$info/o;
	s/\[DISKMODSCRIPTURL\]/$diskModPageURL/o;
	s/\[DISKLISTSCRIPTURL\]/$diskListPageURL/o;
	s/\[str.([^\]]+)\]/MSG_get_ui("$1")/geo;
	print $handle $_;
    }
    close(TEMPLATE);

}
