#!/usr/bin/perl

# author: Kevin K.M. Chiu
#       Cobalt Microserver
# version: 2.0
# date: 7-6-99
#
# Copyright 1999-2000, Cobalt Networks, Inc
# All rights reserved

### START HARDCODED PATHS
my $Template_dir = '/usr/admserv/templates';

my $Debug = (-e "/etc/DEBUG") ? 1 : 0;
$^W = $Debug;
### END HARDCODED PATHS
use Cobalt::Group;
use Cobalt::Locale;
use Cobalt::Vsite;
use Cobalt::User;
use Cobalt::Meta;
use Cobalt::Util;

use Fcntl qw(O_RDWR O_CREAT);
use CGI;
# use strict;
use Locale::gettext;

# global variables
my $q=new CGI;

my $forbiddenURL='/.cobalt/error/forbidden.html';

my $page=$q->param( 'page' );
my $group;

if( $ENV{'PARAM_GROUP'} ) {
    $group=$ENV{'PARAM_GROUP'};
}
else {
    $group=$q->param('group');
    my $user = $ENV{REMOTE_USER};
    unless (user_authok($user, "group", $group)) {
	print "location: $forbiddenURL\n\n";    
	die "Unauthorised access by $user, not a member of $group.\n";
    } 
}

my $siteUserDefaultsScriptURL="/cgi-bin/.cobalt/siteUserDefaults/siteUserDefaults.cgi";
my $siteUserListScriptURL="/cgi-bin/.cobalt/siteUserList/siteUserList.cgi";

my $locale=Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
if( $locale ) { $ENV{ 'LANGUAGE' }=$locale->getLocale(); }

if ($group && ( $page ne 'userdefaults' )) {
    genSiteUserDefaultsPage( \*STDOUT );
}
else { 
    handleSiteUserDefaults(); 
}

1;


#
# event handler for site user defaults page
#
sub handleSiteUserDefaults {
    my $diskQuota=$q->param( 'diskquota' );

    my $obj=Cobalt::Meta->create("type" => "vsite");
    $obj->retrieve($group);

    my $shell=$q->param('shell');
    my $nameGenMode=$q->param('namegenmode');
    my $apop=$q->param('apop');
    my $fpx=$q->param('fpx');
    my $range=$q->param('range');
    
    ($shell eq 'on') ? ($shell = 'on') : ($shell = 'off');
    ($apop eq 'on') ? ($apop = 'on') : ($apop = 'off');
    ($fpx eq 'on') ? ($fpx = 'on') : ($fpx = 'off');
    
    my $info='<IMG SRC="/.cobalt/images/bang" ALIGN=CENTER>';
    my $err=0;
    my $siteDiskQuota=$obj->get("quota");

    if ($range < 1) {
	$info.=eval("\"".Locale::gettext::dgettext("ui", "range_invalid")."\"");
	$err=1;
    }
    # check disk quota
    if( $diskQuota!~/^\d+$/o || $diskQuota==0 ) {
	$info.=eval("\"".Locale::gettext::dgettext("ui", "diskQuotaNotPositive")."\"");
	$err=1;
    }
    elsif( $diskQuota>$siteDiskQuota ) {
	$info.=eval("\"".Locale::gettext::dgettext("ui", "userDiskQuotaTooBig")."\"");
	$err=1;
    }
    else {
	$obj->put("user_quota", $diskQuota);
    }

    unless ($err) {
	$obj->put("user_shell"     => $shell,
		  "user_apop"      => $apop,
		  "user_namegen"   => $nameGenMode,
		  "user_fpx"       => $fpx,
		  "userlist_range" => $range);
	$obj->save;

	print "location: $siteUserListScriptURL?group=$group\n\n";
    } else {
	genSiteUserDefaultsPage( \*STDOUT, $info );
    }
}

#
# argument 0 is the file handle the page goes to
# argument 1 is the information shown to user
#
sub genSiteUserDefaultsPage {
    my ( $handle, $info )=@_;

    my $disabled=eval("\"".Locale::gettext::dgettext("ui", "disabled")."\"");
    # read defaults
    my $obj=Cobalt::Meta->create("type" => "vsite");
    $obj->retrieve($group);

    my $diskQuota=$obj->get("user_quota");

    my ($fpxRow)='<INPUT TYPE=CHECKBOX NAME="fpx" VALUE="on" CHECKED>';
    if (($obj->get("fpx") eq 'on') && ($obj->get("user_fpx") ne 'on')) {
	$fpxRow =~ s/CHECKED//;
    } elsif (($obj->get("fpx") ne 'on')) {
	$fpxRow = $disabled;
    }

    my ($shellRow)='<INPUT TYPE=CHECKBOX NAME="shell" VALUE="on" CHECKED>';
    if (($obj->get("shell") eq 'on') && ($obj->get("user_shell") ne 'on')) {
	$shellRow =~ s/CHECKED//;
    } elsif (($obj->get("shell") ne 'on')) {
	$shellRow = $disabled;
    }

    my ($apopRow)='<INPUT TYPE=CHECKBOX NAME="apop" VALUE="on" CHECKED>';
    if (($obj->get("apop") eq 'on') && ($obj->get("user_apop") ne 'on')) {
	$apopRow =~ s/CHECKED//;
    } elsif (($obj->get("apop") ne 'on')) {
	$apopRow = $disabled;
    }

    my ($rangeRow)='<INPUT TYPE=TEXT SIZE=5 NAME="range" VALUE="' .
	$obj->get("userlist_range") . '">';

    my ($sortRow)='<SELECT NAME="sort">' .
	'<OPTION VALUE="hostname">' . MSG_get_ui("sort_hostname") . '</OPTION>' .
	'<OPTION VALUE="fullname">' . MSG_get_ui("sort_fullname") . '</OPTION>' .
	'</SELECT>';

    my $genUserName=$locale->getProperty( 'genUserName' );
    my ($mode0Checked,$mode1Checked,$mode2Checked);
    my $hideGenStart='<!-- ';
    my $hideGenEnd=' -->';
    if ($genUserName) {
	$hideGenStart='';
	$hideGenEnd='';

	my $nameGenMode=$obj->get("user_namegen");

	# mode 0 is the default
	if ($nameGenMode==1) {
	    $mode1Checked='CHECKED';
	} elsif ($nameGenMode==2) {
	    $mode2Checked='CHECKED';
	} else {
	    $mode0Checked='CHECKED';
	}
    }

    # Read in the template
    open (TEMPLATE, "$Template_dir/siteUserDefaults.tmpl")
	|| die "Cannot read $Template_dir/siteUserDefaults.tmpl: $!\n";

    print "content-type: text/html\nCache-Control: no-cache\n\n";

    while( <TEMPLATE> ) {
	s/\[DISKQUOTA\]/$diskQuota/o;
	s/\[GROUP\]/$group/o;
	s/\[HIDEGENEND\]/$hideGenEnd/o;
	s/\[HIDEGENSTART\]/$hideGenStart/o;
	if( $genUserName ) {
	    s/\[MODE0CHECKED\]/$mode0Checked/o;
	    s/\[MODE1CHECKED\]/$mode1Checked/o;
	    s/\[MODE2CHECKED\]/$mode2Checked/o;
	}
	# s/\[SHELLCHECKED\]/$shellChecked/o;
	s/\[SHELL\]/$shellRow/o;
	s/\[RANGE\]/$rangeRow/o;
	s/\[SORT\]/$sortRow/o;
	s/\[SITEUSERDEFAULTSSCRIPTURL\]/$siteUserDefaultsScriptURL/o;
	s/\[SITEUSERLISTSCRIPTURL\]/$siteUserListScriptURL/o;
	s/\[USER_INFO\]/$info/o;
	s/\[FPX\]/$fpxRow/o;
	s/\[APOP\]/$apopRow/o;

	s/\[str.([^\]]+)\]/Locale::gettext::dgettext("ui", "$1")/geo;

	print $handle $_;
    }
    close TEMPLATE;
}
