#!/usr/bin/perl
#
# Copyright 1999-2000, Cobalt Networks, Inc
# All rights reserved
#
# written for pacifica by Duncan Laurie <duncan@cobaltnet.com>
#

require Cobalt::Vsite;
import Cobalt::Vsite qw(vsite_get_users);
require Cobalt::User;
import Cobalt::User qw(user_authok);
use Cobalt::Util;
require Cobalt::Meta;
require Cobalt::Locale;

# use strict;
use CGI;

my $locale=Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
if( $locale ) { $ENV{ 'LANGUAGE' }=$locale->getLocale(); }

my $forbiddenURL="/.cobalt/error/forbidden.html";
my ($group,$info,$qualify,$low,$q,$sort,$order);
my ($search,$searchby,$searchop,$searchtext);
my (@where);
$q=new CGI;

if ($ENV{'PARAM_GROUP'}) {
    $group=$ENV{'PARAM_GROUP'};
    $info=$ENV{'PARAM_INFO'};
    $low=$ENV{'PARAM_LOW'};
	$sort='name';
} else {
    $group=$q->param('group');
    my $user = $ENV{REMOTE_USER};
    unless (user_authok($user, "group", $group)) {
	print "location: $forbiddenURL\n\n";    
	die "Unauthorised access by $user, not a member of $group.\n";
    }

    $info=$q->param('info');
    $low=$q->param('low');
    if ($q->param('sort')) {
	$sort = $q->param('sort');
	$order = $q->param('order');
    } else {
	$sort = "name";
    }

    if ($q->param('search') eq 'yes') {
	push @where, "AND";
	$searchtext = $q->param('searchtext');
	if ($searchtext) {
	    $search = 1;
	    $searchby = $q->param('searchby');
	    push @where, $searchby;
	    $searchop = $q->param('searchop');
	    if ($searchop eq 'match') {
		$qualify = '.*' . $searchtext . '.*';
		push @where, '~*';
		push @where, $qualify;
	    } elsif ($searchop eq 'nomatch') {
		$qualify = '.*' . $searchtext . '.*';
		push @where, '!~*';
		push @where, $qualify;
	    } else {
		push @where, $searchop;
		push @where, $searchtext;
	    }
	} else {
	    $search = 0;
	}
    }
}

print $q->header(-expires => '-1d');
print '<META HTTP-EQUIV="Pragma" CONTENT="no-cache">';

# global variables
my $URL_siteUserAdd="/cgi-bin/.cobalt/siteUserAdd/siteUserAdd.cgi";
my $PATH_search_users="/usr/admserv/html/.cobalt/search/search_users.html.";
$PATH_search_users .= $locale->getLocale();
my $URL_siteUserList="/cgi-bin/.cobalt/siteUserList/siteUserList.cgi";
my $URL_siteUserDefaults="/cgi-bin/.cobalt/siteUserDefaults/siteUserDefaults.cgi";
my $URL_siteUserEmail="/cgi-bin/.cobalt/siteUserEmail/siteUserEmail.cgi";
my $URL_siteUserMod="/cgi-bin/.cobalt/siteUserMod/siteUserMod.cgi";
my $URL_siteUserDel="/cgi-bin/.cobalt/siteUserDel/siteUserDel.cgi";

genhtml_siteUserList("group" => $group,
		     "info"  => $info,
		     "low"   => $low,
		     "sort"  => $sort,
		     "order" => $order,
		     "search" => $search,
		     "searchby" => $searchby,
		     "searchop" => $searchop,
		     "searchtext" => $searchtext,
		     "where" => \@where);

sub genhtml_siteUserList
# generate the site user list using the specified parameters
#      group = vsite of users to list, set to "" for all sites
#      info = error info to display to the page...
#      keys = fields to grab from the database for each user
#           (eg: "name, fullname, shell, fpx, apop, admin")
#      qualify = any limiting factors (eg: "admin = off")
{
    my %opt = @_;

    my $group = $opt{"group"};
    my $info = $opt{"info"};
    my $low = $opt{"low"};
    my $search = $opt{"search"};
    my @where = @{ $opt{"where"} };
    my $sort = $opt{"sort"};
    my $order = $opt{"order"};

    my ($desc,$lastnamefirst,$settings,$u,@users,$numusers);
    my ($javascript,$temp,$html,$ret,$line);

    # get sort and range values from settings
    my ($count,$sorted);
    if ($group) {
	unshift @where, ("vsite", "=", $group);
	$settings = Cobalt::Meta->create("type" => "vsite");
	$settings->retrieve($group);
	$count=$settings->get("userlist_range");
	$count = ($count < 1) ? 1 : $count;
	$low = 0 unless ($low);
	$numusers = scalar(Cobalt::Vsite::vsite_get_users($group));
    }

    my $spacer = '<FONT COLOR="#FFFFFF">&nbsp;</FONT>';

    # define help text
    my ($modifyUserX,$modifyUserEmail,$removeUser);
    my $help_userlist = MSG_get_ui("userListHelp");
    my $help_fullname = MSG_get_ui("fullNameHelp");
    my $help_username = MSG_get_ui("userNameHelp");
    my $help_access = MSG_get_ui("userAccessHelp");
    my $help_aliases = MSG_get_ui("emailAliasesHelp1");
    my $help_action = MSG_get_ui("actionHelp");
    my $help_confirmRemove = MSG_get_ui("confirmRemoveUser");
    my $str_help = MSG_get_ui("help");
    my $str_saving = MSG_get_ui("saving");
    my $str_userList = MSG_get_ui("userList");
    my $str_username = MSG_get_ui("userName");
    my $str_fullname = MSG_get_ui("fullName");
    my $str_aliases = MSG_get_ui("emailAliases");

    my $str_sort_users_name_asc = MSG_get_ui("sort_users_list_name_asc");
    my $str_sort_users_name_desc = MSG_get_ui("sort_users_list_name_desc");
    my $str_sort_users_fullname_asc = MSG_get_ui("sort_users_list_fullname_asc");
    my $str_sort_users_fullname_desc = MSG_get_ui("sort_users_list_fullname_desc");

    my $searchstr;
    if ($search) {
	$count = "all";
	$low = 0;
	$searchstr = "&search=yes";
	$searchstr .= "&searchby=" . $opt{"searchby"};
	$searchstr .= "&searchop=" . $opt{"searchop"};
	$searchstr .= "&searchtext=" . $opt{"searchtext"};
    }

    my ($sort_name_icon,$sort_fullname_icon);

    my $sort_name_icon_go = "<A HREF=\"$URL_siteUserList?group=$group&low=$low&sort=name$searchstr\" onMouseOver=\"status=\'$str_sort_users_name_asc\'; return true;\"><IMG SRC=\"/.cobalt/images/both_arrow_icon\" BORDER=0 ALT=\"$str_sort_users_name_asc\"></A>";
    my $sort_name_icon_asc = "<A HREF=\"$URL_siteUserList?group=$group&low=$low&sort=name$searchstr\" onMouseOver=\"status=\'$str_sort_users_name_asc\'; return true;\"><IMG SRC=\"/.cobalt/images/up_arrow_icon\" BORDER=0 ALT=\"$str_sort_users_name_asc\"></A>";
    my $sort_name_icon_desc = "<A HREF=\"$URL_siteUserList?group=$group&low=$low&sort=name&order=desc$searchstr\" onMouseOver=\"status=\'$str_sort_users_name_desc\'; return true;\"><IMG SRC=\"/.cobalt/images/down_arrow_icon\" BORDER=0 ALT=\"$str_sort_users_name_desc\"></A>";

    my ($sort_fullname_icon_go, $sort_fullname_icon_asc, $sort_fullname_icon_desc);
    $sort_fullname_icon_go = "<A HREF=\"$URL_siteUserList?group=$group&low=$low&sort=fullname$searchstr\" onMouseOver=\"status=\'$str_sort_users_fullname_asc\'; return true;\"><IMG SRC=\"/.cobalt/images/both_arrow_icon\" BORDER=0 ALT=\"$str_sort_users_fullname_asc\"></A>";
    $sort_fullname_icon_asc = "<A HREF=\"$URL_siteUserList?group=$group&low=$low&sort=fullname$searchstr\" onMouseOver=\"status=\'$str_sort_users_fullname_asc\'; return true;\"><IMG SRC=\"/.cobalt/images/up_arrow_icon\" BORDER=0 ALT=\"$str_sort_users_fullname_asc\"></A>";
    $sort_fullname_icon_desc = "<A HREF=\"$URL_siteUserList?group=$group&low=$low&sort=fullname&order=desc$searchstr\" onMouseOver=\"status=\'$str_sort_users_fullname_desc\'; return true;\"><IMG SRC=\"/.cobalt/images/down_arrow_icon\" BORDER=0 ALT=\"$str_sort_users_fullname_desc\"></A>";

    if ($opt{"order"}) {
	if ($opt{"sort"} eq "name") {
	    $sort_name_icon = $sort_name_icon_asc;
	    $sort_fullname_icon = $sort_fullname_icon_go;
	    $sorted = "&sort=name&order=desc";
	    $str_username = "<b>$str_username</b>";
	} else {
	    $sort_name_icon = $sort_name_icon_go;
	    $sort_fullname_icon = $sort_fullname_icon_asc;
	    $sorted = "&sort=fullname&order=desc";
	    $str_fullname = "<b>$str_fullname</b>";
	}
    } else {
	if ($opt{"sort"} eq "name") {
	    $sort_name_icon = $sort_name_icon_desc;
	    $sort_fullname_icon = $sort_fullname_icon_go;
	    $sorted = "&sort=name";
	    $str_username = "<b>$str_username</b>";
	} else {
	    $sort_name_icon = $sort_name_icon_go;
	    $sort_fullname_icon = $sort_fullname_icon_desc;
	    $sorted = "&sort=fullname";
	    $str_fullname = "<b>$str_fullname</b>";
	}
    }

    # define html for search
    my $search_box = '';
    $ret = FILE_lock_r($PATH_search_users,*SBOX);
    if ($ret) {
	while ($line = <SBOX>) {
	    $line =~ s/\[GROUP\]/$group/o;
	    $search_box .= $line;
	}
	FILE_unlock(*SBOX);
    }

    # define html for some buttons
    my ($button_siteUserDefaults,$button_siteUserAdd,$userlist_title,$buttext);
    my ($button_listNext,$button_listPrev,$button_listFirst,$button_listLast) = ('','','','');
    $button_siteUserDefaults = '<A HREF="' . $URL_siteUserDefaults . "?group=$group" .
	'" onMouseOver="status=\'' . MSG_get_ui("setUserDefaults") . '\'; return true;">' .
	'<IMG SRC="/.cobalt/images/user_defaults_but" ALT="' .
	MSG_get_ui("setUserDefaults") . '" BORDER=0></A>';
    $button_siteUserAdd = '<A HREF="' . $URL_siteUserAdd . "?group=$group" . 
	'" onMouseOver="status=\'' . MSG_get_ui("addUser") .
	'\'; return true;"><IMG SRC="/.cobalt/images/add_user_but" ALT="' .
	MSG_get_ui("addUser") . '" BORDER=0></A>';

    my $button_listNextGray = '<IMG SRC="/.cobalt/images/arrow_next_gray" BORDER="0">';
    my $button_listPrevGray = '<IMG SRC="/.cobalt/images/arrow_prev_gray" BORDER="0">';
    my $button_listLastGray = '<IMG SRC="/.cobalt/images/arrow_last_gray" BORDER="0">';
    my $button_listFirstGray = '<IMG SRC="/.cobalt/images/arrow_first_gray" BORDER="0">';

    my $button_listNext = $button_listNextGray;
    my $button_listPrev = $button_listPrevGray;
    my $button_listLast = $button_listLastGray;
    my $button_listFirst = $button_listFirstGray;

    my $prevlow = $low - $count;
    $prevlow = ($prevlow > 0) ? $prevlow : 0;
    my $nextlow = $low + $count;
    $nextlow = ($nextlow > $numusers) ? $numusers : $nextlow;
    my $numleft = $numusers - $nextlow;
    $numleft = ($numleft > 0) ? $numleft : 0;
    my $lastpage = $numusers - $count;
    $lastpage = ($lastpage > 0) ? $lastpage : 0;

    if ($low > 0) {
	$buttext = MSG_get_ui("userListPrev",$count);
	$button_listPrev = '<A HREF="' . $URL_siteUserList . '?low=' . $prevlow
	    . '&group=' . $group . $sorted . '" onMouseOver="status=\'' . $buttext
	    . '\'; return true;"><IMG SRC="/.cobalt/images/arrow_prev" ALT="'
	    . $buttext . '" BORDER=0></A>';
	$buttext = MSG_get_ui("userListFirst",$count);
	$button_listFirst = '<A HREF="' . $URL_siteUserList . '?low=0&group=' . $group
	    . $sorted . '" onMouseOver="status=\'' . $buttext
	    . '\'; return true;"><IMG SRC="/.cobalt/images/arrow_first" ALT="'
	    . $buttext . '" BORDER=0></A>';
    }
    if ($numleft) {
	my $nextnum = ($count > $numleft) ? $numleft : $count;
	$buttext = MSG_get_ui("userListNext",$nextnum);
	$button_listNext = '<A HREF="' . $URL_siteUserList . '?low=' . $nextlow
	    . '&group=' . $group . $sorted . '" onMouseOver="status=\'' . $buttext
	    . '\'; return true;"><IMG SRC="/.cobalt/images/arrow_next" ALT="'
	    . $buttext . '" BORDER=0></A>';
	$buttext = MSG_get_ui("userListLast",$count);
	$button_listLast = '<A HREF="' . $URL_siteUserList . '?low=' . $lastpage
	    . '&group=' . $group . $sorted . '" onMouseOver="status=\'' . $buttext
	    . '\'; return true;"><IMG SRC="/.cobalt/images/arrow_last" ALT="'
	    . $buttext . '" BORDER=0></A>';
    }
    $button_listPrev = $button_listPrevGray if ($prevlow == 0);
    $button_listNext = $button_listNextGray if ($nextlow == $lastpage);
    $button_listLast = $button_listLastGray if ($nextlow > $lastpage);

    # query the database with acquired settings
    my @keys = ( "vsite", "name", "fullname", "fpx", "shell",
		 "admin", "apop", "aliases", "suspend" );

    # not all languages handle sorting the 'fullname' field
    # so there is a field called 'altname' that can be used
    my $altname = $locale->getProperty("fullname_sort");
    if ("$altname" && ("$sort" eq "fullname")) {
	$sort = "alt";
	push @keys, "name($altname) as alt";
    }
    
    @users = Cobalt::Meta::query("type"  => "users",
				 "keys"  => \@keys,
				 "where" => \@where,
				 "sort"  => "$sort",
				 "order" => "$order",
				 "limit" => [ "$count", "$low" ]);
    my $numfound = scalar(@users);

    if ($search) {
	$button_listNext = $button_listNextGray;
	$button_listPrev = $button_listPrevGray;
	$button_listFirst = $button_listFirstGray;
	$button_listLast = $button_listLastGray;

	if ($numfound == 1) {
	    $userlist_title = MSG_get_ui("search_users_results_1");
	} else {
	    $userlist_title = MSG_get_ui("search_users_results", $numfound);
	}
    } elsif ($numusers > 0) {
	my $ulow = $low + 1;
	my $uhi = ($numusers < $count) ? $numusers : ($low + $count);
	$uhi = ($uhi > $numusers) ? $numusers : $uhi;
	$userlist_title = "$str_userList  ($ulow - $uhi " . MSG_get_ui("of") . " $numusers)";
    } else {
	$userlist_title = $str_userList;
    }

    # define html for some icons
    my $icon_admin = '<IMG SRC="/.cobalt/images/administrator" BORDER=0 ALT="' .
	MSG_get_ui("siteAdmin") . '">';
    my $icon_fpx = '<IMG SRC="/.cobalt/images/counter_icon_small" BORDER=0 ALT="' .
	MSG_get_ui("fpxShowSite") . '">';
    my $icon_shell = '<IMG SRC="/.cobalt/images/telnet_icon_small" BORDER=0 ALT="' .
	MSG_get_ui("shellAccess") . '">';
    my $icon_suspend = '<IMG SRC="/.cobalt/images/suspend_icon_small" BORDER=0 ALT="' .
	MSG_get_ui("suspended_user") . '">';
    my $icon_apop = '<IMG SRC="/.cobalt/images/apop_icon_small" BORDER=0 ALT="' .
	MSG_get_ui("apop_user") . '">';
    my $icon_blank = '<IMG SRC="/.cobalt/images/blank" BORDER=0>';

    # define javascript
    $javascript = <<END;
top.showInfo('$info');

var help_userlist='$help_userlist';
var help_username='$help_username';
var help_fullname='$help_fullname';
var help_access='$help_access';
var help_aliases='$help_aliases';
var help_action='$help_action';

function modify (name) {
    location='${URL_siteUserMod}?username='+name+'&group='+escape('$group')+'&';
}

function modify_email (name) {
    location='${URL_siteUserEmail}?name='+name+'&group='+escape('$group')+'&';
}

function del (userName) {
    if(confirm($help_confirmRemove)) {
	top.showInfo('<IMG SRC=/.cobalt/images/saving> $str_saving', 0);
	location='${URL_siteUserDel}?name='+userName+'&group='+escape('$group')+'&';
    }
}
END
    ;

    print $q->start_html(-title => $str_userList,
			 -BGCOLOR => '#FFFFFF',
			 -script => $javascript);


    print $search_box;

    $html = <<END;
<CENTER>

<TABLE BORDER="0" CELLSPAING="0" BGCOLOR="#FFFFFF" WIDTH="600">
    <TR>
	<TD WIDTH="140" ALIGN="LEFT" NOWRAP>
	    $button_listFirst
	    $button_listPrev
	</TD>
	<TD WIDTH="300" ALIGN="CENTER" NOWRAP>
	    $button_siteUserDefaults
	    $button_siteUserAdd
	</TD>
        <TD WIDTH="140" ALIGN="RIGHT" NOWRAP>
	    $button_listNext
	    $button_listLast
	</TD>
    </TR>
</TABLE>

<TABLE BORDER="1" CELLSPACING="0" WIDTH="600">
    <TR>
	<TH COLSPAN="5" BGCOLOR="#6666CC" ALIGN="CENTER" VALIGN="MIDDLE">
	    <CENTER>
	    <A HREF="javascript: top.showInfo( help_userlist );" onMouseOver="status=' $str_help '; top.showInfo( help_userlist ); return true;"><IMG SRC="/.cobalt/images/question" BORDER="0"></A>
	    <FONT COLOR="#FFFFFF">$userlist_title</FONT>
	    </CENTER>
	</TH>
    </TR>
END
    ;
    print $html;

    if ($numfound == 0) {
	my $noUsers = ($search) ? MSG_get_ui("search_users_nonefound") : MSG_get_ui("noUsers");
	$html = <<END;
<TR>
    <TD BGCOLOR="#FFFFFF" COLSPAN="5"><FONT COLOR="#FFFFFF">&nbsp;</FONT><I>$noUsers</I></TD>
</TR>
END
    ;
    } else {

	$html = <<END;
    <TR>
	<TD WIDTH="160" ALIGN="LEFT" NOWRAP>
	    $spacer<A HREF="javascript: top.showInfo( help_fullname );" onMouseOver="status='$str_help'; top.showInfo( help_fullname ); return true;"><IMG SRC="/.cobalt/images/smallest_help_but" ALIGN="BOTTOM" BORDER="O"></A>
	    <FONT SIZE="+1">$str_fullname</FONT>$sort_fullname_icon
        </TD>
	<TD WIDTH="150" ALIGN="LEFT" NOWRAP>
	    $spacer<A HREF="javascript: top.showInfo( help_username );" onMouseOver="status='$str_help'; top.showInfo(help_username); return true;"><IMG SRC="/.cobalt/images/smallest_help_but" ALIGN="BOTTOM" BORDER="O"></A>
	    <FONT SIZE="+1">$str_username</FONT>$sort_name_icon
        </TD>
	<TD WIDTH="130" ALIGN="LEFT" NOWRAP>
	    $spacer<A HREF="javascript: top.showInfo( help_aliases );" onMouseOver="status='$str_help'; top.showInfo(help_aliases); return true;"><IMG SRC="/.cobalt/images/smallest_help_but" ALIGN="BOTTOM" BORDER="O"></A>
            <FONT SIZE="+1">$str_aliases</FONT>
        </TD>
	<TD WIDTH="80" ALIGN="CENTER" NOWRAP>
	    <A HREF="javascript: top.showInfo( help_access );" onMouseOver="status='$str_help'; top.showInfo(help_access); return true;"><IMG SRC="/.cobalt/images/smallest_help_but" ALIGN="BOTTOM" BORDER="O"></A>
        </TD>
	<TD WIDTH="80" ALIGN="CENTER">
	    <A HREF="javascript: top.showInfo(help_action);" onMouseOver="status='$str_help'; top.showInfo(help_action); return true;"><IMG SRC="/.cobalt/images/smallest_help_but" ALIGN="BOTTOM" BORDER="O"></A>
        </TD>
END
    ;
    }
    print $html;

    # now for each user generate a table row
    my ($fullname,$firstName,$lastName,$username,$name,$aliases,$access);
    foreach (@users) {
	$name = $_->get("name");
	$username = $name;
	$aliases = $_->get("aliases");
	$aliases =~ s/\s+/, /g;
	$fullname = $_->get("fullname");

	if ($_->get("suspend") eq 'on') {
	    $access = "<CENTER> $icon_suspend </CENTER>";
	    $name = "<FONT COLOR=\"#AAAAAA\">$name</FONT>";
	    $fullname = "<FONT COLOR=\"#AAAAAA\">$fullname</FONT>";
	    $aliases = "<FONT COLOR=\"#AAAAAA\">$aliases</FONT>";
	} else {
	    $access = ($_->get("admin") eq 'on') ? $icon_admin : $icon_blank;
	    $access .= ($_->get("shell") eq 'on') ? $icon_shell : $icon_blank;
	    $access .= ($_->get("fpx") eq 'on') ? $icon_fpx : $icon_blank;
	    $access .= ($_->get("apop") eq 'on') ? $icon_apop : $icon_blank;
	}

	$modifyUserX = MSG_get_ui("modifyUserX", $username);
	$modifyUserEmail = MSG_get_ui("modifyUserEmail", $username);
	$removeUser = MSG_get_ui("removeUser", $username);

	$html = <<END;
<TR>
    <TD BGCOLOR="#FFFFFF" VALIGN=MIDDLE ALIGN=LEFT NOWRAP>$spacer $fullname</TD>
    <TD BGCOLOR="#FFFFFF" VALIGN=MIDDLE ALIGN=LEFT NOWRAP>$spacer $name</TD>
    <TD BGCOLOR="#FFFFFF" VALIGN=MIDDLE ALIGN=LEFT>$spacer $aliases</TD>
    <TD BGCOLOR="#FFFFFF" VALIGN=MIDDLE ALIGN=LEFT NOWRAP>$access</TD>
    <TD NOWRAP BGCOLOR="#FFFFFF" VALIGN=MIDDLE ALIGN=CENTER>
        <A HREF="javascript: modify('$username');" onMouseOver="status='$modifyUserX'; return true;"><IMG SRC="/.cobalt/images/modify" ALT="$modifyUserX" BORDER="0"></A>
        <A HREF="javascript: modify_email('$username');" onMouseOver="status='$modifyUserEmail'; return true;"><IMG SRC="/.cobalt/images/edit_email_but" ALT="$modifyUserEmail" BORDER="0"></A>
        <A HREF="javascript: del('$username');" onMouseOver="status='$removeUser'; return true;"><IMG SRC="/.cobalt/images/delete" ALT="$removeUser" BORDER="0"></A>
        </CENTER>
    </TD>
</TR>

END
    ;
	print $html;
    }

    print "</TABLE>";
    print $q->end_html;
	
}

