#!/usr/bin/perl

# author: Kevin K.M. Chiu
#       Cobalt Microserver
# revised by: Harris Vaegan-Lloyd
#	Cobalt Networks
# version: 2.0
# date: 6-9-98
#
# version 1.0 supports getstats with GIFgraph
# version 2.0 supports analog without GIFgraph
# version 2.1 supports mail and ftp logs.
#
# Copyright 1998-1999, Cobalt Networks, Inc
# All rights reserved

### START HARDCODED PATHS
my $AdminPages_dir = '/usr/admserv/html/.cobalt';
my $Template_dir = '/usr/admserv/templates';

my $Debug = (-e "/etc/DEBUG") ? 1 : 0;
$^W = $Debug;
### END HARDCODED PATHS
use Cobalt::Utility;
use Cobalt::Vsite;
my %opts;

use strict;
use Getopt::Std;
getopt( 'stho' , \%opts);

# read arguments

my $SiteName = $opts{'s'} || 'main';

my $ReportType = $opts{'t'} ||
	die "We require a log type -t [web|mail|ftp]\n";

my $OutputDir = $opts{'o'} ||
	die "We require an output directory\n";

# We need a hash of fqdns to group names later.. Generate it now as we'll use
# it all over the place..

my %fqdn_to_group;
foreach my $vsite (vsite_list()) {
	$fqdn_to_group{$$vsite[1]} = $$vsite[2];
}

#
# global variables
#
#my $mainPagePath="$AdminPages_dir/webUsage/$id"."index.html";
my $mainPagePath="$OutputDir/webUsage.html";

my $mostRequestedPagePath="$OutputDir/mostRequested.html";
my $mostRequestedPageURL="mostRequested.html";

my $periodicPagePath="$OutputDir/periodic.html";
my $periodicPageURL="periodic.html";

my $domainPagePath="$OutputDir/domain.html";
my $domainPageURL="domain.html";

my $referrerPagePath="$OutputDir/referrer.html";
my $referrerPageURL="referrer.html";

my $browserPagePath="$OutputDir/browser.html";
my $browserPageURL="browser.html";

my $graphDataPath="/usr/admserv/html/.cobalt/report/$SiteName.$ReportType.dat";
my $graphDataURL="/.cobalt/report/$SiteName.$ReportType.dat";

my $webLogScriptURL="/cgi-bin/.cobalt/webLog/webLog.cgi?group=". $fqdn_to_group{$opts{'s'}} . "&service=" . $opts{'t'};

my $domains="$AdminPages_dir/webUsage/domains";


# the number of request data shown in request report
my $maxRequestShown=100;

# max number of separate domains shown
my $maxDomainShown=20;

# max number of separate referrers shown
my $maxReferrerShown=100;

# max number of separate browsers shown
my $maxBrowserShown=20;

# maximum width for html bars
my $maxBarWidth=300;

# the back button
my $back="<A HREF=\"webUsage.html\" onMouseOver=\"status='Back'; return true;\"><IMG SRC=\"/.cobalt/images/back_but\" WIDTH=\"63\" HEIGHT=\"20\" ALIGN=\"BOTTOM\" BORDER=\"0\"></A>";

if ($ReportType eq 'mail') {
	my @links = (	[ '[str.messagesPerAddress]',  $mostRequestedPageURL ],
					[ '[str.downloadLog]' , $webLogScriptURL  ],
					[ '[str.outgoingMailDomain]', $browserPageURL ] );
	push @links, [ '[str.incomingMailDomain]', $referrerPageURL ] if $SiteName eq 'main';
	genMainPage( \*STDIN,
				$mainPagePath,
				"mailUsageSum.tmpl",
				 \@links);
	genPage(	\*STDIN ,
				'[str.messagesPerAddress]',
				$mostRequestedPagePath,
				$mostRequestedPageURL,
				'webStats.tmpl',
				'^r\s+RrBb\s+(\d+)\s+(\S+)\s+(\d+)\s+(\S+)\s+(.+)',
				20,
				\&to_mail);
	genPage(	\*STDIN ,
				'[str.incomingMailDomain]',
				$referrerPagePath,
				$referrerPageURL,
				'webStats.tmpl',
				'^f\s+RrBb\s+(\d+)\s+(\S+)\s+(\d+)\s+(\S+)\s+(.+)',
				20,
				# Doing this is kind of stupid if this is not the main site.
				\&to_domain);
	genPage(	\*STDIN,
				'[str.outgoingMailDomain]',
				$browserPagePath,
				$browserPageURL,
				'webStats.tmpl',
				'^B\s+RrBb\s+(\d+)\s+(\S+)\s+(\d+)\s+(\S+)\s+(.+)',
				20,
				\&to_domain);
	genGraphData( \*STDIN, $graphDataPath );

} elsif ( $ReportType eq 'ftp' ) {
	my @links = (	[ '[str.mostRequested]',  $mostRequestedPageURL ],
					[ '[str.downloadLog]' , $webLogScriptURL  ] );
	push @links, [ '[str.usageDomain]', $browserPageURL ] if $SiteName eq 'main';
	genMainPage( \*STDIN,
				$mainPagePath,
				"ftpUsageSum.tmpl",
				 \@links);
	genPage(	\*STDIN ,
				'[str.mostRequested]',
				$mostRequestedPagePath,
				$mostRequestedPageURL,
				'webStats.tmpl',
				'^r\s+RrBb\s+(\d+)\s+(\S+)\s+(\d+)\s+(\S+)\s+(.+)',
				20,
				\&to_ftp_url);
	genPage(	\*STDIN,
				'[str.usageDomain]',
				$browserPagePath,
				$browserPageURL,
				'webStats.tmpl',
				'^B\s+RrBb\s+(\d+)\s+(\S+)\s+(\d+)\s+(\S+)\s+(.+)',
				20,
				\&to_domain)  if $SiteName eq 'main';
	genGraphData( \*STDIN, $graphDataPath );
} elsif ( $ReportType eq 'web' ) {
	my @links = (	[ '[str.webMostRequested]',  $mostRequestedPageURL ],
					[ '[str.BCC_036]' , $referrerPageURL ],
					[ '[str.accessByDomain]' , $domainPageURL ],
					[ '[str.BCC_032]' , $browserPageURL ],
					[ '[str.downloadLog]' , $webLogScriptURL  ] );
	genMainPage( \*STDIN,
				$mainPagePath,
				"webUsageSum.tmpl",
				 \@links);
	genPage(	\*STDIN ,
				'[str.webMostRequested]',
				$mostRequestedPagePath,
				$mostRequestedPageURL,
				'webStats.tmpl',
				'^r\s+RrBb\s+(\d+)\s+(\S+)\s+(\d+)\s+(\S+)\s+(.+)',
				20,
				($SiteName eq 'main') ? ( \&main_to_http_url ) : (\&to_http_url) );
	genPage(	\*STDIN ,
				'[str.BCC_036]',
				$referrerPagePath,
				$referrerPageURL,
				'webStats.tmpl',
				'^f\s+RrBb\s+(\d+)\s+(\S+)\s+(\d+)\s+(\S+)\s+(.+)',
				20,
				\&null_echo);
	genPage(	\*STDIN,
				'[str.BCC_032]',
				$browserPagePath,
				$browserPageURL,
				'webStats.tmpl',
				'^B\s+RrBb\s+(\d+)\s+(\S+)\s+(\d+)\s+(\S+)\s+(.+)',
				20,
				\&null_echo);
	genPage(	\*STDIN,
				'[str.accessByDomain]',
				$domainPagePath,
				$domainPageURL,
				'webStats.tmpl',
				'^S\s+RrBb\s+(\d+)\s+(\S+)\s+(\d+)\s+(\S+)\s+(.+)',
				20,
				\&null_echo);
	genGraphData( \*STDIN, $graphDataPath );
} else {
	die "Invalid log type: $ReportType\n";
}
# must be after genGraphPage
genPeriodicPage($periodicPagePath, $graphDataPath, $periodicPageURL, $graphDataURL);

1;

sub to_mail {
	# Get's handed a "/email@domain.com/" and turns it into a mailto tag.
	my $email = shift;
	$email =~ s#/(.*)/#$1#;
	return "<A mailto:$email>$email</A>";
}

sub null_echo {
	my $echo = shift;
	return "</B> $echo </B>";
}

sub to_domain {
	# Gets handed a "http://domain.com" and returns a bold domains name.
	my $domain = shift;
	$domain =~ s#http://(.*)/#$1#;
	$domain ||= 'Local';
	return "<B> $domain </B>";
}

sub to_http_url {
	my $line = shift;
	# Gets handed a "/foo/bar.gz" and returns a "ftp://xx/foo/bar.gz" url.
	return '<A HREF="http://'.$SiteName.$line.'"/>http://'.$SiteName.$line.'</A>';
}

sub main_to_http_url {
	my $line = shift;
	# Gets handed a /www.site.com/foo/bar.html and returns an appropriate link
	$line =~ m/\/([^\/]+)\/(.*)/;
	return "<A HREF=\"http://$1/$2\">http://$1/$2</A>";
}

sub to_ftp_url {
	my $line = shift;
	# Gets handed a "/home/sites/xx/foo/bar.gz" and returns a "ftp://xx/foo/bar.gz" url.
	# Strip out the /home/sites/sitename/ bit of the url..
	$line =~ m/\/home\d*\/sites\/([^\/]+)\/(.*)/;
	my $site_name = ((vsite_get_bygroup($1))[1]);
	return "<A HREF=\"ftp://$site_name/$2\">ftp://$site_name/$2</A>";
}

sub genMainPage {
    my ( $reportHandle )=shift;
	my ( $pagePath, $template, $pages ) = @_;


	my $links;
    my $line;

	foreach ( @{$pages} ) {
		$links .= "<TR><TD WIDTH=\"434\"><UL><A HREF=\"$$_[1]\"><IMG SRC=\"/.cobalt/images/chart_icon_small\" WIDTH=20 ALIGN=\"BOTTOM\" HEIGHT=20>$$_[0]</A></UL></TD></TR>\n";
	}

    # parse general summary
    my $genDate;
    my $coverage;
    my $totalRequestNum=0;
    my $HTMLRequestNum=0;
    my $badRequestNum=0;
    my $fileNum=0;
    my $hostNum=0;
    my $byteNum=0;
    while( $line=<$reportHandle> ) {
	if( $line!~/^x\s+(\S+)\s+(\S+)(\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+))?/ ) {
	    seek( $reportHandle, -length( $line ), 1 );
	    last;
	}

	if( $1 eq 'PS' ) { $genDate="$2/$4/$5 $6:$7"; }
	elsif( $1 eq 'FR' ) { $coverage="$2/$4/$5 $6:$7"; }
	elsif( $1 eq 'LR' ) { $coverage.=" - $2/$4/$5 $6:$7"; }
	elsif( $1 eq 'SR' ) { $totalRequestNum=$2; }
	elsif( $1 eq 'PR' ) { $HTMLRequestNum=$2; }
	elsif( $1 eq 'FL' ) { $badRequestNum=$2; }
	elsif( $1 eq 'NF' ) { $fileNum=$2; }
	elsif( $1 eq 'NH' ) { $hostNum=$2; }
	elsif( $1 eq 'BT' ) { $byteNum=$2; }
    }

	# Fill in some blank values if we had a blank
	# stats file. Generally a sign of no activity.

	$genDate ||=  '[str.noActivity]';
	$coverage ||= '[str.noActivity]';

    # calculate percentage
    my $HTMLShare;
    my $badShare;
    if( $totalRequestNum!=0 ) {
		$HTMLShare=int( 1000*$HTMLRequestNum/$totalRequestNum )/10;
		$badShare=int( 1000*$badRequestNum/$totalRequestNum )/10;
    } else {
		$HTMLShare=0;
		$badShare=0;
		#sauce_cp("$Template_dir/webUsageNoReport.tmpl",$pagePath);
		#warn "Did not generate a report";
		#exit 0;
    }

    open( MAIN, ">$pagePath" ) ||
		die "Could not write to $pagePath: $!\n";

    # Read in the template
    open( TEMPLATE, "$Template_dir/$template" )
        || die "Cannot read $Template_dir/$template: $!\n";

    while( <TEMPLATE> ) {
	s/\[GENDATE\]/$genDate/o;
	s/\[COVERAGE\]/$coverage/o;
	s/\[HTMLREQUESTNUM\]/$HTMLRequestNum/o;
	s/\[HTMLSHARE\]/$HTMLShare/o;
	s/\[BADREQUESTNUM\]/$badRequestNum/o;
	s/\[BADSHARE\]/$badShare/o;
	s/\[TOTALREQUESTNUM\]/$totalRequestNum/o;
	s/\[HOSTNUM\]/$hostNum/o;
	s/\[FILENUM\]/$fileNum/o;
	s/\[BYTENUM\]/$byteNum/o;

	s/\[LINKS\]/$links/o;

	print MAIN;
    }

    close( TEMPLATE );
    close( MAIN );
}


#
# no arguments
#
sub genPeriodicPage {
	my ($pagePath, $dataPath, $pageUrl, $dataUrl) = @_;
    if( ! -s $dataPath ) { hideLink( $pageUrl ); }
    else {
	open( PERIODIC, ">$pagePath" ) ||
		die "Could not open $pagePath: $!\n";

	# Read in the template
	open( TEMPLATE, "$Template_dir/webUsagePeriodic.tmpl" )
	    || die "Cannot read $Template_dir/webUsagePeriodic.tmpl: $!\n";

	while( <TEMPLATE> ) {
	    s/\[GRAPHDATAURL\]/$dataUrl/o;
	    s/\[BACK\]/$back/o;

	    print PERIODIC;
	}

	close( TEMPLATE );
	close( PERIODIC );
    }
}

#
# argument 0 is the file handle of the report generated by analog
#
sub genGraphData {
    my $reportHandle=shift;
	my $dataPath = shift;
    my $line;

    my $startDailySection=0;
    my $startHourlySection=0;
    my $start5MinuteSection=0;

    open( GRAPHDATA, ">$dataPath" ) ||
		die "Could not open $dataPath: $!\n";

    select GRAPHDATA;

    while( <$reportHandle> ) {
	# skip empty lines
	if( /^$/ ) { next; }

	# skip until useful data
	if( !/^[dh5]/ ) { next; }

	# daily data
	if( /^d\s+P\s+(\d+)/ ) {
	    # 1st line in the section?
	    if( !$startDailySection ) {
		$startDailySection=1;
		print "-\n";
	    }

	    # print data
	    print "$1\n";
	}

	# hourly data
	if( /^h\s+P\s+(\d+)/ ) {
	    # 1st line in the section?
	    if( !$startHourlySection ) {
		$startHourlySection=1;
		print "-\n";
	    }

	    # print data
	    print "$1\n";
	}

	if( /^5\s+P\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)/ ) {
	    # 1st line in the section?
	    if( !$start5MinuteSection ) {
		$start5MinuteSection=1;
		# show the date
		print "-\n$2 $3 $4 $5 $6\n";
	    }

	    # print data
	    print "$1\n";
	}
    }

    close( GRAPHDATA );
}


#
# argument 0 is the URL
#
sub hideLink {
    my ( $link )=@_;

    my $tmpPage="/tmp/webUsage$$.html";
	if ( -l $tmpPage ) {
		die "$tmpPage is a link. Possible hack attempt\n";
	}
    # Read in main page
    open( IN, "<$mainPagePath" )
	|| die "Cannot read $mainPagePath: $!\n";

    open( OUT, ">$tmpPage" );

    while( <IN> ) {
		# grey out the link
		if ( /<A HREF="$link">(.*)<\/A>/ ) {
			s/<A HREF="$link">(.*)<\/A>/<FONT COLOR=#CCCCCC>$1<\/FONT>/;
		}

	print OUT;
    }
    close( IN );
    close( OUT );

    # update
    system( "mv $tmpPage $mainPagePath" );
}



sub genPage
# Arguments: A refrence to the filehandle of the report up to the appropriate section.
#			 A path to put the completed page.
#			 A url for the selcted page.
#			 A Template to use for the page.
#			 A regexp which will return $1 .. $5 where
#				$1 = Arbitary integer.
#				$2 = percentage for $1
#				$3 = arbitary integer.
#				$4 = percentage of $3
#				$5 = Name to use.
#			 The maximum number of entries to show.
#			 A refrence to a function to use to convert $5 into the actual
#			 displayed form.  e.g. wrap it with a http:// | ftp:// |
#			 mailto: style url or convert domain names to a nice format
#			 e.g. .to -> Tonga.

{ my $reportHandle=shift; my ($pageTitle, $pagePath, $pageUrl, $template,
$regexp, $maxShown, $function) = @_;

	# To later check for and continue throught the section of the analog
	# stats file that lists floors we will need to get the first two
	# letters of the regexp out (It makes sense if you know what the file
	# looks like.)
	#
	# Bassically at the beginning of each section there is a line that
	# specifies the floors for the analog report, we have to read, then
	# ignore this line.

	my $floor_regex = substr($regexp, 0, 2);
	
    open( PAGE, ">$pagePath") ||
		die "Could not open $pagePath: $!\n";

    my @entrys;

	print STDERR "Searching for $regexp\n" if $Debug;
    while( my $line=<$reportHandle> ) {
		print STDERR "Got to $line\n" if $Debug;
		if( $line!~/$regexp/) {
			if ($line!~/$floor_regex/) {
		    	seek( $reportHandle, -length( $line ), 1 );
		    	last;
			} else {
				next;
			}
		}
		# If the identifier begins with a charachter that it not [ and has
		# one or more charachter after that (e.g. not null. as a match with
		# $floor_regex will cause.
		if( $5 && substr($5, 0, 1) ne '[' ) {
		    push @entrys, 	[
			&$function($5), 		# The entry name..
		   	$1, 					# The first stat (Requests.)
		   	sprintf('%.1f', $2), 	# Second stats (Percentage of requests.)
			int($3/1024), 			# Third stat (Bytes.)
			sprintf('%.1f', $4) ];	# Fourth stat (Percentage of bytes.)
		}
	}

   	my $showNum=( ( $#entrys+1 )< $maxShown )?( $#entrys+1):$maxShown;

    if( $showNum < 1 ) {
		print STDERR "Hiding link to $pageUrl\n" if $Debug;
		hideLink( $pageUrl );
		return 1;
	}

	# Read in the template
	open( TEMPLATE, "$Template_dir/$template" )
	    || die "Cannot read $Template_dir/$template: $!\n";

	my $requestRows;

	my $i;
	foreach $i ( 0..$showNum-1 ) {
	    my $barWidth=int( $maxBarWidth*${$entrys[$i]}[ 2 ]/100 );

	    $requestRows .= <<EOF;
    <TR>
        <TD>
            $entrys[ $i ][ 0 ]
        </TD>
        <TD>
            $entrys[ $i ][ 1 ] ($entrys[ $i ][ 2 ]\%)
        </TD>
        <TD>
            $entrys[ $i ][ 3 ]k ($entrys[ $i ][ 4 ]\%)
        </TD>
        <TD>
EOF


	    if( $barWidth>0 ) {
			$requestRows .= "            <IMG SRC=\"/.cobalt/images/blue_color\" WIDTH=$barWidth HEIGHT=10>\n";
	    } else {
			$requestRows.="            <FONT COLOR=#FFFFFF>&nbsp;</FONT>\n";
		}

	    $requestRows .= "        </TD>\n    </TR>\n";
	}

	#print $requestRows;

	while ( <TEMPLATE> ) {
	    s/\[TITLE\]/$pageTitle/o;
	    s/\[ROWS\]/$requestRows/o;
	    s/\[SHOWNUM\]/$showNum/o;
	    s/\[BACK\]/$back/o;
	    print PAGE;
	}

	close( TEMPLATE );

    close( PAGE ); 
}
